@@ -7,6 +7,7 @@ import re
 import shutil
 import sys
 import time
+import urllib.parse
 from datetime import datetime
 
 import dataframe_image as dfi
@@ -31,6 +32,7 @@ from .changefiles import (
     get_prev_hour,
     get_prev_year_dates,
     in_local_timezone,
+    last_days_count,
     previous_day,
     previous_month,
     previous_week,
@@ -41,6 +43,8 @@ from .changesets import ChangesetToolKit
 
 users_temp = {}
 users = {}
+summary_interval = {}
+summary_interval_temp = {}
 hashtag_changesets = {}
 countries_changesets = {}
 
@@ -51,7 +55,7 @@ countries_df = gpd.read_file(
 
 
 def collect_changefile_stats(
-    user, uname, changeset, version, tags, osm_type, osm_obj_nodes=None
+    user, uname, changeset, version, tags, osm_type, timestamp, osm_obj_nodes=None
 ):
     tags_to_collect = list(additional_tags) if additional_tags else None
     if version == 1:
@@ -60,12 +64,74 @@ def collect_changefile_stats(
         action = "modify"
     if version == 0:
         action = "delete"
+    timestamp = timestamp.strftime("%Y-%m-%d")
+    len_feature = 0
+    if length and osm_obj_nodes:
+        try:
+            len_feature = osmium.geom.haversine_distance(osm_obj_nodes)
+        except Exception as ex:
+            # print("WARNING: way  incomplete." % w.id)
+            pass
+    if summary:
+        summary_interval.setdefault(
+            timestamp,
+            {
+                "timestamp": timestamp,
+                "users": 0,
+                "changesets": 0,
+                "nodes": {"create": 0, "modify": 0, "delete": 0},
+                "ways": {"create": 0, "modify": 0, "delete": 0},
+                "relations": {"create": 0, "modify": 0, "delete": 0},
+                "poi": {"create": 0, "modify": 0},
+            },
+        )
+
+        # for changeset count
+        summary_interval_temp.setdefault(timestamp, {"changesets": [], "users": []})
+        if changeset not in summary_interval_temp[timestamp]["changesets"]:
+            summary_interval_temp[timestamp]["changesets"].append(changeset)
+        summary_interval[timestamp]["changesets"] = len(
+            summary_interval_temp[timestamp]["changesets"]
+        )
+        # for users count
+        if user not in summary_interval_temp[timestamp]["users"]:
+            summary_interval_temp[timestamp]["users"].append(user)
+        summary_interval[timestamp]["users"] = len(
+            summary_interval_temp[timestamp]["users"]
+        )
+        # for nodes , ways relations count
+        summary_interval[timestamp][osm_type][action] += 1
+        # for poi
+        if osm_type == "nodes" and tags and action != "delete":
+            summary_interval[timestamp]["poi"][action] += 1
+
+        # for user supplied tags
+        if tags_to_collect and action != "delete" and tags:
+            for tag in tags_to_collect:
+                summary_interval[timestamp].setdefault(tag, {"create": 0, "modify": 0})
+                if tag in tags:
+                    summary_interval[timestamp][tag][action] += 1
+        # for length calculation
+        if length:
+            for t in length:
+                summary_interval[timestamp].setdefault(f"{t}_create_len", 0)
+                if t in tags and action != "modify" and action != "delete":
+                    summary_interval[timestamp][f"{t}_create_len"] += round(len_feature)
+        # for all tags
+        if all_tags:
+            summary_interval[timestamp].setdefault("tags_create", {})
+            summary_interval[timestamp].setdefault("tags_modify", {})
+            if tags:
+                for key, value in tags:
+                    if action != "delete":
+                        summary_interval[timestamp][f"tags_{action}"].setdefault(key, 0)
+                        summary_interval[timestamp][f"tags_{action}"][key] += 1
+
     if user in users:
-        users[user]["name"] = uname
-        users[user]["uid"] = user
         if changeset not in users_temp[user]["changesets"]:
             users_temp[user]["changesets"].append(changeset)
         users[user]["changesets"] = len(users_temp[user]["changesets"])
+
         if hashtags or changeset:
             try:
                 for ch in hashtag_changesets[changeset]["countries"]:
@@ -109,12 +175,7 @@ def collect_changefile_stats(
                     and action != "modify"
                     and action != "delete"
                 ):
-                    try:
-                        len_feature = osmium.geom.haversine_distance(osm_obj_nodes)
-                        users[user][f"{t}_create_len"] += round(len_feature)
-                    except Exception as ex:
-                        # print("WARNING: way  incomplete." % w.id)
-                        pass
+                    users[user][f"{t}_create_len"] += round(len_feature)
 
     else:
         users[user] = {
@@ -160,7 +221,7 @@ def collect_changefile_stats(
 
 
 def calculate_stats(
-    user, uname, changeset, version, tags, osm_type, osm_obj_nodes=None
+    user, uname, changeset, version, tags, osm_type, timestamp, osm_obj_nodes=None
 ):
     if (hashtags and country) or hashtags:  # intersect with changesets
         if (
@@ -172,22 +233,43 @@ def calculate_stats(
                     and changeset in countries_changesets.keys()
                 ):
                     collect_changefile_stats(
-                        user, uname, changeset, version, tags, osm_type, osm_obj_nodes
+                        user,
+                        uname,
+                        changeset,
+                        version,
+                        tags,
+                        osm_type,
+                        timestamp,
+                        osm_obj_nodes,
                     )
             else:
                 if changeset in hashtag_changesets.keys():
                     collect_changefile_stats(
-                        user, uname, changeset, version, tags, osm_type, osm_obj_nodes
+                        user,
+                        uname,
+                        changeset,
+                        version,
+                        tags,
+                        osm_type,
+                        timestamp,
+                        osm_obj_nodes,
                     )
     elif country:
         if len(countries_changesets) > 0:
             if changeset in countries_changesets.keys():
                 collect_changefile_stats(
-                    user, uname, changeset, version, tags, osm_type, osm_obj_nodes
+                    user,
+                    uname,
+                    changeset,
+                    version,
+                    tags,
+                    osm_type,
+                    timestamp,
+                    osm_obj_nodes,
                 )
     else:  # collect everything
         collect_changefile_stats(
-            user, uname, changeset, version, tags, osm_type, osm_obj_nodes
+            user, uname, changeset, version, tags, osm_type, timestamp, osm_obj_nodes
         )
 
 
@@ -267,14 +349,16 @@ class ChangefileHandler(osmium.SimpleHandler):
         super(ChangefileHandler, self).__init__()
 
     def node(self, n):
-        if n.timestamp >= start_date_utc and n.timestamp <= end_date_utc:
+        if n.timestamp >= start_date_utc and n.timestamp < end_date_utc:
             version = n.version
             if n.deleted:
                 version = 0
-            calculate_stats(n.uid, n.user, n.changeset, version, n.tags, "nodes")
+            calculate_stats(
+                n.uid, n.user, n.changeset, version, n.tags, "nodes", n.timestamp
+            )
 
     def way(self, w):
-        if w.timestamp >= start_date_utc and w.timestamp <= end_date_utc:
+        if w.timestamp >= start_date_utc and w.timestamp < end_date_utc:
             version = w.version
             if w.deleted:
                 version = 0
@@ -285,15 +369,18 @@ class ChangefileHandler(osmium.SimpleHandler):
                 version,
                 w.tags,
                 "ways",
+                w.timestamp,
                 w.nodes if length else None,
             )
 
     def relation(self, r):
-        if r.timestamp >= start_date_utc and r.timestamp <= end_date_utc:
+        if r.timestamp >= start_date_utc and r.timestamp < end_date_utc:
             version = r.version
             if r.deleted:
                 version = 0
-            calculate_stats(r.uid, r.user, r.changeset, version, r.tags, "relations")
+            calculate_stats(
+                r.uid, r.user, r.changeset, version, r.tags, "relations", r.timestamp
+            )
 
 
 def process_changefiles(url):
@@ -312,7 +399,7 @@ def process_changefiles(url):
 
 def process_changesets(url):
     # print(f"Processing {url}")
-    file_path = get_file_path_from_url(url, "changesets")
+    file_path = get_file_path_from_url(url, "changeset")
     handler = ChangesetHandler()
     handler.apply_file(file_path[:-3])
     # print(f"Finished {url}")
@@ -414,8 +501,9 @@ def parse_args():
 
     parser.add_argument(
         "--url",
-        default="https://planet.openstreetmap.org/replication/minute",
-        help="Your public OSM Change Replication URL ",
+        nargs="+",
+        default=["https://planet.openstreetmap.org/replication/minute"],
+        help="Your public list of OSM Change Replication URL , 'minute,hour,day' option by default will translate to planet replciation url. You can supply multiple urls for geofabrik country updates , Url should not have trailing / at the end",
     )
 
     parser.add_argument(
@@ -448,6 +536,12 @@ def parse_args():
         help="Extract stats for Last hour",
         default=False,
     )
+    parser.add_argument(
+        "--days",
+        type=int,
+        default=None,
+        help="N nof of last days to extract , for eg if 3 is supplied script will generate stats for last 3 days",
+    )
     parser.add_argument(
         "--charts",
         action="store_true",
@@ -457,7 +551,7 @@ def parse_args():
     parser.add_argument(
         "--summary",
         action="store_true",
-        help="Produces Summary.md file with summary of Run",
+        help="Produces Summary.md file with summary of Run and also a summary.csv which will have summary of stats per day",
         default=False,
     )
     parser.add_argument(
@@ -481,9 +575,9 @@ def parse_args():
         default=False,
     )
     parser.add_argument(
-        "--store",
+        "--temp",
         action="store_true",
-        help="Stores downloaded osm files to machine itself to run different stats generation again, Will be useful if you are doing multiple analysis so that script will not download the files that are already downloaded",
+        help="Deletes downloaded osm files from machine after processing is done , if you want to run osmsg on same files again keep this option turn off",
         default=False,
     )
 
@@ -511,6 +605,7 @@ def parse_args():
 
 
 def main():
+    print("OSMSG : Waking up ......")
     args = parse_args()
     if args.start_date:
         start_date = strip_utc(
@@ -524,10 +619,13 @@ def main():
             or args.last_month
             or args.last_year
             or args.last_hour
+            or args.days
         ):
             pass
         else:
-            print("ERR: Supply start_date")
+            print(
+                "ERR: Supply start_date or extraction parameters such as last_day , last_hour"
+            )
             sys.exit()
 
     if args.end_date:
@@ -556,6 +654,7 @@ def main():
     global country
     global changeset
     global exact_lookup
+    global summary
 
     all_tags = args.all_tags
     additional_tags = args.tags
@@ -565,18 +664,53 @@ def main():
     changeset = args.changeset
     exact_lookup = args.exact_lookup
     length = args.length
+    summary = args.summary
 
-    if "geofabrik" in args.url.lower():
-        if args.username is None:
-            args.username = os.environ.get("OSM_USERNAME")
-        if args.password is None:
-            args.password = os.environ.get("OSM_PASSWORD")
+    if args.url:
+        args.url = list(set(args.url))  # remove duplicates
+        for url in args.url:
+            if urllib.parse.urlparse(url).scheme == "":
+                # The URL is not valid
+                if url == "minute":
+                    args.url = ["https://planet.openstreetmap.org/replication/minute"]
+                elif url == "hour":
+                    args.url = ["https://planet.openstreetmap.org/replication/hour"]
+                elif url == "day":
+                    args.url = ["https://planet.openstreetmap.org/replication/day"]
+                else:
+                    print(f"Invalid input for urls {url}")
+                    sys.exit()
+            if url.endswith("/"):
+                print(f"{url} should not end with trailing /")
+                sys.exit()
 
-        if not (args.username and args.password):
-            assert (
-                args.username and args.password
-            ), "OSM username and password are required for geofabrik url"
-        cookies = auth(args.username, args.password)
+        if any("geofabrik" in url.lower() for url in args.url):
+            if args.username is None:
+                args.username = os.environ.get("OSM_USERNAME")
+            if args.password is None:
+                args.password = os.environ.get("OSM_PASSWORD")
+
+            if not (args.username and args.password):
+                assert (
+                    args.username and args.password
+                ), "OSM username and password are required for geofabrik url"
+            cookies = auth(args.username, args.password)
+
+    count = sum(
+        [
+            args.last_hour,
+            args.last_year,
+            args.last_month,
+            args.last_day,
+            args.last_week,
+            bool(args.days),
+        ]
+    )
+    if count > 1:
+        print(
+            "Error: only one of --last_hour, --last_year, --last_month, --last_day, --last_week, or --days should be specified."
+        )
+        sys.exit()
 
     if args.last_hour:
         start_date, end_date = get_prev_hour(args.timezone)
@@ -592,6 +726,12 @@ def main():
     if args.last_week:
         start_date, end_date = previous_week(args.timezone)
 
+    if args.days:
+        if args.days > 0:
+            start_date, end_date = last_days_count(args.timezone, args.days)
+        else:
+            print(f"Error : {args.days} should be greater than 0")
+            sys.exit()
     if args.read_from_metadata:
         if os.path.exists(args.read_from_metadata):
             with open(args.read_from_metadata, "r") as openfile:
@@ -632,8 +772,7 @@ def main():
             f"Processing Changeset from {strip_utc(Changeset.sequence_to_timestamp(changeset_start_seq),args.timezone)} to {strip_utc(Changeset.sequence_to_timestamp(changeset_end_seq),args.timezone)}"
         )
 
-        temp_path = os.path.join(os.getcwd(), "temp/changesets")
-        print(os.path.exists(temp_path))
+        temp_path = os.path.join(os.getcwd(), "temp/changeset", "changesets")
         if not os.path.exists(temp_path):
             os.makedirs(temp_path)
 
@@ -646,7 +785,7 @@ def main():
             # Use `map` to apply the `download osm files` function to each element in the `urls` list
             for _ in tqdm(
                 executor.map(
-                    lambda x: download_osm_files(x, mode="changesets", cookies=cookies),
+                    lambda x: download_osm_files(x, mode="changeset", cookies=cookies),
                     changeset_download_urls,
                 ),
                 total=len(changeset_download_urls),
@@ -674,93 +813,115 @@ def main():
         end_seq_timestamp = Changeset.sequence_to_timestamp(changeset_end_seq)
         if end_date > end_seq_timestamp:
             end_date = strip_utc(end_seq_timestamp, args.timezone)
+    for url in args.url:
+        print(f"Changefiles : Generating Download Urls Using {url}")
+        (
+            download_urls,
+            server_ts,
+            start_seq,
+            end_seq,
+            start_seq_url,
+            end_seq_url,
+        ) = get_download_urls_changefiles(start_date, end_date, url, args.timezone)
+        if server_ts < end_date:
+            print(
+                f"Warning : End date data is not available at server, Changing to latest available date {server_ts}"
+            )
+            end_date = server_ts
+            if start_date >= server_ts:
+                print("Err: Data is not available after start date ")
+                sys.exit()
+        global end_date_utc
+        global start_date_utc
 
-    print(f"Changefiles : Generating Download Urls Using {args.url}")
-    (
-        download_urls,
-        server_ts,
-        start_seq,
-        end_seq,
-        start_seq_url,
-        end_seq_url,
-    ) = get_download_urls_changefiles(start_date, end_date, args.url, args.timezone)
-    if server_ts < end_date:
+        start_date_utc = start_date.astimezone(dt.timezone.utc)
+        end_date_utc = end_date.astimezone(dt.timezone.utc)
         print(
-            f"Warning : End date data is not available at server, Changing to latest available date {server_ts}"
+            f"Final UTC Date time to filter stats : {start_date_utc} to {end_date_utc}"
         )
-        end_date = server_ts
-        if start_date >= server_ts:
-            print("Err: Data is not available after start date ")
-            sys.exit()
-    global end_date_utc
-    global start_date_utc
+        # Use the ThreadPoolExecutor to download the images in parallel
+        max_workers = os.cpu_count() if not args.workers else args.workers
+        print(f"Using {max_workers} Threads")
 
-    start_date_utc = start_date.astimezone(dt.timezone.utc)
-    end_date_utc = end_date.astimezone(dt.timezone.utc)
-    print(f"Final UTC Date time to filter stats : {start_date_utc} to {end_date_utc}")
-    temp_path = os.path.join(os.getcwd(), "temp/changefiles")
-    if not os.path.exists(temp_path):
-        os.makedirs(temp_path)
+        print("Downloading Changefiles")
+        with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
+            # Use `map` to apply the `download osm files` function to each element in the `urls` list
+            for _ in tqdm(
+                executor.map(
+                    lambda x: download_osm_files(
+                        x, mode="changefiles", cookies=cookies
+                    ),
+                    download_urls,
+                ),
+                total=len(download_urls),
+                unit_scale=True,
+                unit="changefiles",
+                leave=True,
+            ):
+                pass
+        print("Processing Changefiles")
 
-    # Use the ThreadPoolExecutor to download the images in parallel
-    max_workers = os.cpu_count() if not args.workers else args.workers
-    print(f"Using {max_workers} Threads")
+        with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
+            # Use `map` to apply the `download_image` function to each element in the `urls` list
+            # executor.map(process_changefiles, download_urls)
+            for _ in tqdm(
+                executor.map(process_changefiles, download_urls),
+                total=len(download_urls),
+                unit_scale=True,
+                unit="changefiles",
+                leave=True,
+            ):
+                pass
 
-    print("Downloading Changefiles")
-    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
-        # Use `map` to apply the `download osm files` function to each element in the `urls` list
-        for _ in tqdm(
-            executor.map(
-                lambda x: download_osm_files(x, mode="changefiles", cookies=cookies),
-                download_urls,
-            ),
-            total=len(download_urls),
-            unit_scale=True,
-            unit="changefiles",
-            leave=True,
-        ):
-            pass
-    print("Processing Changefiles")
-
-    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
-        # Use `map` to apply the `download_image` function to each element in the `urls` list
-        # executor.map(process_changefiles, download_urls)
-        for _ in tqdm(
-            executor.map(process_changefiles, download_urls),
-            total=len(download_urls),
-            unit_scale=True,
-            unit="changefiles",
-            leave=True,
-        ):
-            pass
-
-        # executor.shutdown(wait=True)
-    print("Changefiles Processing Finished")
+            # executor.shutdown(wait=True)
+        print(f"Changefiles Processing Finished using {url}")
     os.chdir(os.getcwd())
-    if not args.store:
+    if args.temp:
         shutil.rmtree("temp")
     if len(users) >= 1:
         # print(users)
         if args.all_tags:
             for user in users:
-                users[user]["tags_create"] = json.dumps(
-                    dict(
-                        sorted(
-                            users[user]["tags_create"].items(),
-                            key=lambda item: item[1],
-                            reverse=True,
+                for action in ["create", "modify"]:
+                    users[user][f"tags_{action}"] = json.dumps(
+                        dict(
+                            sorted(
+                                users[user][f"tags_{action}"].items(),
+                                key=lambda item: item[1],
+                                reverse=True,
+                            )
                         )
                     )
-                )
-                users[user]["tags_modify"] = json.dumps(
-                    dict(
-                        sorted(
-                            users[user]["tags_modify"].items(),
-                            key=lambda item: item[1],
-                            reverse=True,
+            if summary:
+                for timestamp in summary_interval:
+                    for action in ["create", "modify"]:
+                        summary_interval[timestamp][f"tags_{action}"] = json.dumps(
+                            dict(
+                                sorted(
+                                    summary_interval[timestamp][
+                                        f"tags_{action}"
+                                    ].items(),
+                                    key=lambda item: item[1],
+                                    reverse=True,
+                                )
+                            )
                         )
-                    )
-                )
+        summary_df = pd.json_normalize(list(summary_interval.values()))
+        summary_df = summary_df.assign(
+            changes=summary_df["nodes.create"]
+            + summary_df["nodes.modify"]
+            + summary_df["nodes.delete"]
+            + summary_df["ways.create"]
+            + summary_df["ways.modify"]
+            + summary_df["ways.delete"]
+            + summary_df["relations.create"]
+            + summary_df["relations.modify"]
+            + summary_df["relations.delete"]
+        )
+        summary_df.insert(3, "map_changes", summary_df["changes"], True)
+        summary_df = summary_df.drop(columns=["changes"])
+
+        summary_df = summary_df.sort_values("timestamp", ascending=True)
 
         df = pd.json_normalize(list(users.values()))
 
@@ -881,6 +1042,7 @@ def main():
             create_charts(df)
 
         if args.summary:
+            summary_df.to_csv(f"summary.csv", index=False)
             created_sum = (
                 df["nodes.create"] + df["ways.create"] + df["relations.create"]
             )
